@@ -1,13 +1,13 @@
 import datetime
 import json
-from urllib.parse import urlparse
 
-import fsspec
 from google.cloud import datastore
 
+from cirrocumulus.database_api import load_dataset_schema, get_email_domain
 from .invalid_usage import InvalidUsage
 
 DATASET = 'Dataset'
+CAT_NAME = 'Cat_Name'
 DATASET_FILTER = 'Dataset_Filter'
 USER = 'User'
 
@@ -19,8 +19,32 @@ class FirestoreDatastore:
 
     def server(self):
         client = self.datastore_client
-        server = dict(email=client.project + '@appspot.gserviceaccount.com')
-        return server
+        return dict(mode='server', email=client.project + '@appspot.gserviceaccount.com')
+
+    def category_names(self, email, dataset_id):
+        dataset_id = int(dataset_id)
+        self.__get_key_and_dataset(email, dataset_id, False)
+        client = self.datastore_client
+        query = client.query(kind=CAT_NAME)
+        query.add_filter('dataset_id', '=', dataset_id)
+        results = []
+        for result in query.fetch():
+            results.append(result)
+        return results
+
+    def upsert_category_name(self, email, category, dataset_id, original_name, new_name):
+        dataset_id = int(dataset_id)
+        self.__get_key_and_dataset(email, dataset_id, False)
+        client = self.datastore_client
+        key = client.key(CAT_NAME, str(dataset_id) + '-' + str(category) + '-' + str(original_name))
+
+        if new_name == '':
+            client.delete(key)
+        else:
+            entity = datastore.Entity(key=key)
+            entity.update(dict(category=category, dataset_id=dataset_id, original=original_name, new=new_name))
+            client.put(entity)
+            return entity.id
 
     def user(self, email):
         client = self.datastore_client
@@ -36,10 +60,19 @@ class FirestoreDatastore:
         client = self.datastore_client
         query = client.query(kind=DATASET)
         query.add_filter('readers', '=', email)
-
         results = []
         for result in query.fetch():
             results.append({'id': result.id, 'name': result['name'], 'owner': email in result['owners']})
+        domain = get_email_domain(email)
+        if domain is not None:
+            query = client.query(kind=DATASET)
+            query.add_filter('readers', '=', domain)
+            unique_ids = set()
+            for result in results:
+                unique_ids.add(result['id'])
+            for result in query.fetch():
+                if result.id not in unique_ids:
+                    results.append({'id': result.id, 'name': result['name'], 'owner': email in result['owners']})
         return results
 
     def dataset_filters(self, email, dataset_id):
@@ -99,7 +132,9 @@ class FirestoreDatastore:
         if dataset is None:
             raise InvalidUsage('Please provide a valid id', 400)
         readers = dataset.get('readers')
-        if email not in readers:
+
+        domain = get_email_domain(email)
+        if email not in readers and domain not in readers:
             raise InvalidUsage('Not authorized', 403)
         if ensure_owner and email not in dataset['owners']:
             raise InvalidUsage('Not authorized', 403)
@@ -111,15 +146,19 @@ class FirestoreDatastore:
         client.delete(key)
 
     def get_dataset(self, email, dataset_id, ensure_owner=False):
-        key, dataset = self.__get_key_and_dataset(email, dataset_id, ensure_owner)
+        _, dataset = self.__get_key_and_dataset(email, dataset_id, ensure_owner)
+        dataset['id'] = dataset.id
         return dataset
 
     def upsert_dataset(self, email, dataset_id, dataset_name, url, readers):
         client = self.datastore_client
-        if dataset_id is not None:
+        if dataset_id is not None:  # only owner can update
             key, dataset = self.__get_key_and_dataset(email, dataset_id, True)
         else:
             dataset = datastore.Entity(client.key(DATASET), exclude_from_indexes=['url', 'precomputed'])
+            user = client.get(client.key(USER, email))
+            if 'importer' not in user or not user['importer']:
+                raise InvalidUsage('Not authorized', 403)
         readers = set(readers)
         if email in readers:
             readers.remove(email)
@@ -127,13 +166,10 @@ class FirestoreDatastore:
         update_dict = {'name': dataset_name,
                        'readers': list(readers),
                        'url': url}
-        if url.endswith('.json'):
-            pr = urlparse(url)
-            fs = fsspec.filesystem(pr.scheme if not pr.scheme == '' else 'file')
-            with fs.open(url) as s:
-                s = json.load(s)
-                update_dict['precomputed'] = s.get('precomputed', False)
-                update_dict['nObs'] = s.get('nObs')
+        json_schema = load_dataset_schema(url)
+        if json_schema is not None:
+            update_dict['precomputed'] = json_schema.get('precomputed', False)
+            update_dict['shape'] = json_schema.get('shape')
 
         if dataset_id is None:  # new dataset
             update_dict['owners'] = [email]
