@@ -1,31 +1,24 @@
-import {IconButton, Menu, Tooltip} from '@material-ui/core';
+import {Divider, IconButton, Menu, Tooltip} from '@material-ui/core';
 import AppBar from '@material-ui/core/AppBar';
 import Button from '@material-ui/core/Button';
-import Chip from '@material-ui/core/Chip';
-import Grid from '@material-ui/core/Grid';
-import Link from '@material-ui/core/Link';
 import MenuItem from '@material-ui/core/MenuItem';
 import Select from '@material-ui/core/Select';
 import {withStyles} from '@material-ui/core/styles';
-import Switch from '@material-ui/core/Switch';
 import Tab from '@material-ui/core/Tab';
 import Tabs from '@material-ui/core/Tabs';
 import Toolbar from '@material-ui/core/Toolbar';
 import AccountCircle from '@material-ui/icons/AccountCircle';
+import HelpIcon from '@material-ui/icons/Help';
 import MoreVertIcon from '@material-ui/icons/MoreVert';
 import React from 'react';
 import {connect} from 'react-redux';
 import {
     DELETE_DATASET_DIALOG,
-    downloadSelectedIds,
     EDIT_DATASET_DIALOG,
-    getDatasetFilterArray,
+    HELP_DIALOG,
     IMPORT_DATASET_DIALOG,
     login,
     logout,
-    removeDatasetFilter,
-    SAVE_DATASET_FILTER_DIALOG,
-    setCombineDatasetFilters,
     setDataset,
     setDialog,
     setMessage,
@@ -116,19 +109,9 @@ class AppHeader extends React.PureComponent {
         this.setState({moreMenuOpen: false});
     };
 
-    onDatasetFilterChipDeleted = (name) => {
-        this.props.removeDatasetFilter(name);
-    };
 
-    onDatasetFilterCleared = () => {
-        this.props.removeDatasetFilter(null);
-    };
-    onDatasetFilterSaved = () => {
-        this.props.handleDialog(SAVE_DATASET_FILTER_DIALOG);
-    };
-
-    handleCombineDatasetFilters = (event) => {
-        this.props.handleCombineDatasetFilters(event.target.checked ? 'or' : 'and');
+    handleHelp = () => {
+        this.props.handleDialog(HELP_DIALOG);
     };
 
 
@@ -139,19 +122,26 @@ class AppHeader extends React.PureComponent {
         this.setState({moreMenuOpen: true, moreMenuAnchorEl: event.currentTarget});
     };
 
-    copyLink = () => {
-        const {dataset, embeddings, features, groupBy, datasetFilter, interpolator, markerOpacity, unselectedMarkerOpacity, dotPlotData} = this.props;
+    copyLink = (event) => {
+
+        const {dataset, embeddings, searchTokens, datasetFilter, interpolator, markerOpacity, unselectedMarkerOpacity, dotPlotData} = this.props;
         let linkText = window.location.protocol + '//' + window.location.host;
 
         let json = {
             dataset: dataset.id,
-            embeddings: embeddings
+            embeddings: embeddings.map(embedding=>{
+                if(embedding.bin) {
+                    embedding = Object.assign({}, embedding);
+                    delete embedding._bin;
+                    return embedding;
+                }else {
+                    return {name:embedding.name, dimensions:embedding.dimensions}
+                }
+
+            })
         };
-        if (features.length > 0) {
-            json.features = features;
-        }
-        if (groupBy.length > 0) {
-            json.groupBy = groupBy;
+        if (searchTokens.length > 0) {
+            json.q = searchTokens;
         }
 
         let datasetFilterJson = {};
@@ -187,24 +177,40 @@ class AppHeader extends React.PureComponent {
             json.colorScheme = interpolator.name;
         }
         linkText += '?q=' + JSON.stringify(json);
-        const el = document.createElement('textarea');
-        el.value = linkText;
-        el.setAttribute('readonly', '');
-        el.style.position = 'absolute';
-        el.style.left = '-9999px';
-        document.body.appendChild(el);
-        el.select();
-        el.focus();
+        const container = document.activeElement;
+        const fakeElem = document.createElement('textarea');
+        const isRTL = document.documentElement.getAttribute('dir') == 'rtl';
+        fakeElem.style.fontSize = '12pt';
+        fakeElem.style.border = '0';
+        fakeElem.style.padding = '0';
+        fakeElem.style.margin = '0';
+        // Move element out of screen horizontally
+        fakeElem.style.position = 'absolute';
+        fakeElem.style[isRTL ? 'right' : 'left'] = '-9999px';
+        fakeElem.setAttribute('readonly', '');
+        // Move element to the same position vertically
+        let yPosition = window.pageYOffset || document.documentElement.scrollTop;
+        fakeElem.style.top = yPosition + 'px';
+        fakeElem.value = linkText;
+        container.appendChild(fakeElem);
+
+        fakeElem.select();
+        fakeElem.setSelectionRange(0, fakeElem.value.length);
+
         document.execCommand('copy');
-        document.body.removeChild(el);
+
+        const fakeHandlerCallback = (event) => {
+            document.activeElement.blur();
+            window.getSelection().removeAllRanges();
+            container.removeChild(fakeElem);
+            container.removeEventListener('click', fakeHandlerCallback);
+        };
         this.props.setMessage('Link copied');
+        container.addEventListener('click', fakeHandlerCallback);
         this.setState({moreMenuOpen: false});
+
     };
 
-    handleSelectedCellsClick = (event) => {
-        event.preventDefault();
-        this.props.downloadSelectedIds();
-    };
 
     handleLogout = () => {
         this.setState({userMenuOpen: false});
@@ -220,42 +226,34 @@ class AppHeader extends React.PureComponent {
     };
     handleSettings = (event) => {
         this.props.handleDialog(EDIT_DATASET_DIALOG);
+        this.setState({moreMenuOpen: false});
     };
 
     handleDelete = (event) => {
         this.props.handleDialog(DELETE_DATASET_DIALOG);
+        this.setState({moreMenuOpen: false});
     };
 
     render() {
         const {
-            dataset, loadingApp, email, datasetChoices, selection, classes, serverInfo, combineDatasetFilters,
-            datasetFilter, tab, user
+            dataset, loadingApp, email, datasetChoices, selection, classes, serverInfo, tab, user
         } = this.props;
         const shape = dataset != null && dataset.shape != null ? dataset.shape : [0, 0];
         const hasSelection = dataset != null && shape[0] > 0 && !isNaN(selection.count);
         const showNumberOfCells = !hasSelection && dataset != null && !(selection.count > 0) && shape[0] > 0 && (selection.count !== shape[0]);
-        let datasetFilters = getDatasetFilterArray(datasetFilter);
-        const showMoreMenu = (email != null && user.importer) || dataset != null;
-        const datasetFilterKeys = [];
-        let isBrushing = false;
-        datasetFilters.forEach(f => {
-            if (typeof f[0] === 'object') {
-                isBrushing = true;
-            } else {
-                datasetFilterKeys.push(f[0]);
-            }
-        });
-        if (isBrushing) {
-            datasetFilterKeys.push('selection');
-        }
 
 
+        const showNewDataset = user != null && user.importer;
+        const showEditDeleteDataset = dataset !== null && dataset.owner;
+        const showMoreMenu = showNewDataset || dataset != null;
+
         return (
 
             <AppBar position="fixed" color="default" className={classes.appBar}>
                 <Toolbar variant="dense">
-                    {email != null &&
-                    <Select
+
+
+                    {datasetChoices.length > 0 && <Select
                         style={{marginRight: 6}}
                         disableUnderline={true}
                         displayEmpty={true}
@@ -265,17 +263,16 @@ class AppHeader extends React.PureComponent {
                             name: 'dataset',
                             id: 'dataset-id',
                         }}
-                    > {datasetChoices.length > 0 && datasetChoices.length !== 1 &&
-                    <MenuItem key="" value="" disabled>
-                        Choose a dataset
-                    </MenuItem>}
+                    >
+                        <MenuItem key="" value="" disabled>
+                            Choose a dataset
+                        </MenuItem>
                         {datasetChoices.map(dataset => <MenuItem
                             key={dataset.id} value={dataset.id}>{dataset.name}</MenuItem>)}
                     </Select>}
 
                     <div className={"cirro-condensed"} style={{display: 'inline-block'}}>
-                        {hasSelection && (<Link title="Download selected ids" href="#"
-                                                onClick={this.handleSelectedCellsClick}>{intFormat(selection.count)}</Link>)}
+                        {hasSelection && intFormat(selection.count)}
                         {hasSelection && ' / ' + intFormat(shape[0]) + ' cells'}
                         {showNumberOfCells && intFormat(shape[0]) + ' cells'}
                     </div>
@@ -298,51 +295,11 @@ class AppHeader extends React.PureComponent {
                         {/*    style={{display: 'inline', marginRight: 20}}>Cirro</h3>*/}
 
 
-                        <div style={{display: 'inline-block', marginLeft: '10px'}}>
-                            {datasetFilters.length > 0 && <div style={{display: 'inline-block'}}>FILTER:</div>}
-                            {datasetFilterKeys.map(key => {
-                                return <Chip
-                                    size="small"
-                                    onDelete={() => {
-                                        this.onDatasetFilterChipDeleted(key);
-                                    }}
-                                    style={{marginRight: 2}}
-                                    key={key}
-                                    label={key}
-                                    variant={'outlined'}
-                                />;
-                            })}
-
-
-                            {datasetFilters.length > 0 &&
-                            <div style={{display: 'inline-block', marginLeft: '10px'}}>
-                                <Button size="small" color={'primary'}
-                                        onClick={this.onDatasetFilterCleared}>Clear</Button>
-                                <Button size="small" color={'primary'}
-                                        onClick={this.onDatasetFilterSaved}>Save</Button>
-                            </div>}
-
-
-                            {datasetFilters.length > 0 &&
-                            <div style={{display: 'inline-block', marginLeft: '10px'}}>
-                                <Grid component="label" container alignItems="center" spacing={0}>
-                                    <Grid item>AND</Grid>
-                                    <Grid item>
-                                        <Switch
-                                            size="small"
-                                            checked={combineDatasetFilters === 'or'}
-                                            onChange={this.handleCombineDatasetFilters}
-                                        />
-                                    </Grid>
-                                    <Grid item>OR</Grid>
-                                </Grid>
-                            </div>}
-                        </div>
                     </div>
                     <div style={{marginLeft: 'auto'}}>
 
                         {showMoreMenu && <Tooltip title={'More'}>
-                            <IconButton style={{marginLeft: 50}} aria-label="Menu" aria-haspopup="true"
+                            <IconButton aria-label="Menu" aria-haspopup="true"
                                         onClick={this.handleMoreMenuOpen}>
                                 <MoreVertIcon/>
                             </IconButton>
@@ -359,28 +316,30 @@ class AppHeader extends React.PureComponent {
                                                    horizontal: 'right',
                                                }} open={this.state.moreMenuOpen}
                                                onClose={this.handleMoreMenuClose}>
-                            {email != null && user.importer &&
-                            <MenuItem onClick={this.handleImportDataset}>
-                                Import Dataset
-                            </MenuItem>}
-                            {dataset !== null && dataset.owner &&
-                            <MenuItem onClick={this.handleDelete}>Delete Dataset</MenuItem>}
-                            {dataset !== null && dataset.owner &&
-                            <MenuItem onClick={this.handleSettings}>Edit Dataset</MenuItem>}
-                            {dataset != null &&
-                            <MenuItem onClick={this.copyLink}>Copy Link
+                            {showNewDataset && <MenuItem onClick={this.handleImportDataset}>
+                                New Dataset
                             </MenuItem>}
+
+                            {showEditDeleteDataset && <MenuItem onClick={this.handleSettings}>Edit Dataset</MenuItem>}
+                            {showEditDeleteDataset && <MenuItem onClick={this.handleDelete}>Delete Dataset</MenuItem>}
+                            {(showNewDataset || showEditDeleteDataset) && dataset!=null && <Divider/>}
+                            {dataset != null && <MenuItem onClick={this.copyLink}>Copy Link </MenuItem>}
                         </Menu>}
+                        <Tooltip title={'Help'}>
+                            <IconButton aria-label="Help"
+                                        onClick={this.handleHelp}>
+                                <HelpIcon/>
+                            </IconButton>
+                        </Tooltip>
 
-
-                        {email != null &&
+                        {email != null && email!==''&&
                         <Tooltip title={email}>
-                            <IconButton style={{marginLeft: 50}} aria-label="Menu" aria-haspopup="true"
+                            <IconButton aria-label="Menu" aria-haspopup="true"
                                         onClick={this.handleUserMenuOpen}>
                                 <AccountCircle/>
                             </IconButton>
                         </Tooltip>}
-                        {email != null &&
+                        {email != null && email!==''&&
                         <Menu id="menu-user"
                               anchorEl={this.state.userMenuAnchorEl}
                               anchorOrigin={{
@@ -411,9 +370,9 @@ class AppHeader extends React.PureComponent {
 const mapStateToProps = state => {
     return {
         dataset: state.dataset,
-        features: state.features,
+
         embeddings: state.embeddings,
-        groupBy: state.groupBy,
+        searchTokens: state.searchTokens,
         binSummary: state.binSummary,
         binValues: state.binValues,
         combineDatasetFilters: state.combineDatasetFilters,
@@ -457,15 +416,6 @@ const mapDispatchToProps = (dispatch, ownProps) => {
         },
         handleDialog: (value) => {
             dispatch(setDialog(value));
-        },
-        handleCombineDatasetFilters: (value) => {
-            dispatch(setCombineDatasetFilters(value));
-        },
-        downloadSelectedIds: () => {
-            dispatch(downloadSelectedIds());
-        },
-        removeDatasetFilter: (filter) => {
-            dispatch(removeDatasetFilter(filter));
         }
     };
 };
