@@ -34,6 +34,7 @@ class EditDatasetDialog extends React.PureComponent {
         };
     }
 
+
     componentDidMount() {
         if (this.props.dataset != null) {
             fetch(API + '/dataset?id=' + this.props.dataset.id,
@@ -41,6 +42,7 @@ class EditDatasetDialog extends React.PureComponent {
                     method: 'GET',
                     headers: {'Authorization': 'Bearer ' + getIdToken()},
                 }).then(result => result.json()).then(datasetInfo => {
+                    console.log(datasetInfo)
                 let readers = datasetInfo.readers;
                 let myIndex = readers.indexOf(this.props.email);
                 if (myIndex !== -1) {
@@ -53,6 +55,7 @@ class EditDatasetDialog extends React.PureComponent {
                     readers: readers.join(', '),
                 });
             }).catch(err => {
+                console.log(err)
                 this.props.handleError('Unable to retrieve dataset details. Please try again.');
                 this.props.handleCancel();
             });
@@ -114,17 +117,18 @@ class EditDatasetDialog extends React.PureComponent {
                         value={this.state.url}
                         onChange={this.onUrlChanged}
                         margin="dense"
-                        helperText={"Please ensure that " + this.props.serverEmail + " has \"Storage Object Viewer\" permissions"}
-                        label="Dataset URL (gs://my_bucket/my_dataset.parquet)"
+                        helperText={this.props.serverInfo.serverEmail ? "Please ensure that " + this.props.serverInfo.serverEmail + " has \"Storage Object Viewer\" permissions" : ''}
+                        label={"Dataset URL (" + (this.props.serverInfo.serverEmail ? "gs://my_bucket/my_dataset" : "/Users/foo/my_dataset") + ")"}
                         fullWidth
                     />}
 
-                    {!this.state.loading &&
+                    {!this.state.loading && this.props.serverInfo.clientId !== '' &&
                     <TextField
                         value={this.state.readers}
                         onChange={this.onEmailChanged}
                         margin="dense"
-                        label="Comma separated list of emails to share with"
+                        label="Share"
+                        helperText="Enter comma separated list of emails"
                         fullWidth
                         multiline
                     />}
@@ -147,7 +151,7 @@ class EditDatasetDialog extends React.PureComponent {
 const mapStateToProps = state => {
     return {
         dataset: state.dialog === EDIT_DATASET_DIALOG ? state.dataset : null,
-        serverEmail: state.serverInfo.email,
+        serverInfo: state.serverInfo,
         email: state.email
     };
 };
