@@ -1,13 +1,15 @@
 import {Switch} from '@material-ui/core';
+import MuiAccordionPanel from '@material-ui/core/Accordion';
+import MuiAccordionPanelDetails from '@material-ui/core/AccordionDetails';
+import MuiAccordionPanelSummary from '@material-ui/core/AccordionSummary';
 import Box from '@material-ui/core/Box';
 import Button from '@material-ui/core/Button';
 import Checkbox from '@material-ui/core/Checkbox';
+import Chip from '@material-ui/core/Chip';
 import Divider from '@material-ui/core/Divider';
-import MuiExpansionPanel from '@material-ui/core/ExpansionPanel';
-import MuiExpansionPanelDetails from '@material-ui/core/ExpansionPanelDetails';
-import MuiExpansionPanelSummary from '@material-ui/core/ExpansionPanelSummary';
 import FormControl from '@material-ui/core/FormControl';
 import FormControlLabel from '@material-ui/core/FormControlLabel';
+import Grid from '@material-ui/core/Grid';
 import IconButton from '@material-ui/core/IconButton';
 import Input from '@material-ui/core/Input';
 import InputLabel from '@material-ui/core/InputLabel';
@@ -19,48 +21,77 @@ import MenuItem from '@material-ui/core/MenuItem';
 import Select from '@material-ui/core/Select';
 import withStyles from '@material-ui/core/styles/withStyles';
 import TextField from '@material-ui/core/TextField';
+import Tooltip from '@material-ui/core/Tooltip';
+import CloudDownloadIcon from '@material-ui/icons/CloudDownload';
 import DeleteIcon from '@material-ui/icons/Delete';
+import HighlightOffIcon from '@material-ui/icons/HighlightOff';
+import SaveIcon from '@material-ui/icons/Save';
 import React from 'react';
 import {connect} from 'react-redux';
 import {
     deleteDatasetFilter,
+    diffExp,
+    downloadSelectedIds,
     exportDatasetFilters,
+    getDatasetFilterArray,
     getEmbeddingKey,
     handleBrushFilterUpdated,
+    handleCategoricalNameChange,
     handleColorChange,
     handleDimensionFilterUpdated,
     handleMeasureFilterUpdated,
     openDatasetFilter,
+    removeDatasetFilter,
+    SAVE_DATASET_FILTER_DIALOG,
     setBinSummary,
     setBinValues,
-    setFeatures,
+    setChartSize,
+    setCombineDatasetFilters,
+    setDialog,
     setInterpolator,
     setMarkerOpacity,
     setMarkerOpacityUI,
     setNumberOfBins,
     setNumberOfBinsUI,
     setPointSize,
+    setSearchTokens,
     setSelectedEmbedding,
     setUnselectedMarkerOpacity,
     setUnselectedMarkerOpacityUI
 } from './actions';
-
-import AutocompleteSelect from './AutocompleteSelect';
+import AutocompleteVirtualized from './AutocompleteVirtualized';
 import CategoricalLegend from './CategoricalLegend';
 import ColorSchemeLegendWrapper from './ColorSchemeLegendWrapper';
 import ColorSchemeSelector from './ColorSchemeSelector';
+import {intFormat} from './formatters';
+import {splitSearchTokens} from './util';
 
+const pointSizeOptions = [{value: 0.25, label: '25%'}, {value: 0.5, label: '50%'}, {
+    value: 0.75,
+    label: '75%'
+}, {value: 1, label: '100%'}, {value: 1.5, label: '150%'}, {value: 2, label: '200%'}, {
+    value: 3,
+    label: '300%'
+}, {value: 4, label: '400%'}];
+const gallerySizeOptions = [{value: 400, label: 'Small'}, {value: 500, label: 'Medium'}, {
+    value: 600,
+    label: 'Large'
+}];
+const summaryOptions = [
+    {value: 'max', label: 'Maximum'},
+    {value: 'mean', label: 'Mean'},
+    {value: 'sum', label: 'Sum'}];
 const styles = theme => ({
     root: {
         display: 'flex',
-        flexWrap: 'wrap',
+        flexWrap: 'nowrap',
         width: '100%',
-        'flex-direction': 'column',
+        flexDirection: 'column'
     },
     formControl: {
         display: 'block',
-        margin: theme.spacing(1),
         minWidth: 200,
+        margin: theme.spacing(1),
     },
     select: {
         minWidth: 200,
@@ -68,7 +99,7 @@ const styles = theme => ({
 });
 
 
-const ExpansionPanel = withStyles({
+const Accordion = withStyles({
     root: {
         border: '1px solid rgba(0, 0, 0, .125)',
         boxShadow: 'none',
@@ -83,9 +114,9 @@ const ExpansionPanel = withStyles({
         },
     },
     expanded: {},
-})(MuiExpansionPanel);
+})(MuiAccordionPanel);
 
-const ExpansionPanelSummary = withStyles({
+const AccordionPanelSummary = withStyles({
     root: {
         backgroundColor: 'rgba(0, 0, 0, .03)',
         borderBottom: '1px solid rgba(0, 0, 0, .125)',
@@ -101,13 +132,13 @@ const ExpansionPanelSummary = withStyles({
         },
     },
     expanded: {},
-})(MuiExpansionPanelSummary);
+})(MuiAccordionPanelSummary);
 
-const ExpansionPanelDetails = withStyles(theme => ({
+const AccordionPanelDetails = withStyles(theme => ({
     root: {
         padding: 0,
     },
-}))(MuiExpansionPanelDetails);
+}))(MuiAccordionPanelDetails);
 
 
 class EmbedForm extends React.PureComponent {
@@ -128,6 +159,19 @@ class EmbedForm extends React.PureComponent {
         this.props.handlePointSize(event.target.value);
     };
 
+    onFeaturesChange = (event, value) => {
+        this.props.handleSearchTokens(value, 'X');
+    };
+
+    onObservationsChange = (event, value) => {
+        this.props.handleSearchTokens(value, 'obs');
+    };
+
+    onFeatureSetsChange = (event, value) => {
+        this.props.handleSearchTokens(value, 'featureSet');
+    };
+
+
     onMarkerOpacityKeyPress = (event) => {
         if (event.key === 'Enter') {
             let opacity = parseFloat(event.target.value);
@@ -170,6 +214,13 @@ class EmbedForm extends React.PureComponent {
         }
     };
 
+
+    onChartSizeChange = (event) => {
+        const value = event.target.value;
+        this.props.handleChartSize(value);
+
+    };
+
     onBinSummaryChange = (event) => {
         const value = event.target.value;
         this.props.handleBinSummary(value);
@@ -203,9 +254,9 @@ class EmbedForm extends React.PureComponent {
     };
 
     handleEmbeddingsChange = (event) => {
-
         const embeddings = event.target.value;
         const selection = [];
+
         embeddings.forEach(embedding => {
             if (!embedding.precomputed) {
                 embedding = Object.assign(embedding, {
@@ -217,26 +268,64 @@ class EmbedForm extends React.PureComponent {
             }
             selection.push(embedding);
         });
+
+
         this.props.handleEmbeddings(selection);
     };
 
+    handleSelectedCellsClick = (event) => {
+        event.preventDefault();
+        this.props.downloadSelectedIds();
+    };
+    handleDiffExp = (event) => {
+        event.preventDefault();
+        this.props.handleDiffExp();
+    };
+
+    onDatasetFilterChipDeleted = (name) => {
+        this.props.removeDatasetFilter(name);
+    };
+
+    onDatasetFilterCleared = () => {
+        this.props.removeDatasetFilter(null);
+    };
+    onDatasetFilterSaved = () => {
+        this.props.handleDialog(SAVE_DATASET_FILTER_DIALOG);
+    };
+
+    handleCombineDatasetFilters = (event) => {
+        this.props.handleCombineDatasetFilters(event.target.checked ? 'or' : 'and');
+    };
 
     render() {
         const {
-            numberOfBinsUI, interpolator, binValues, binSummary, embeddings, classes, datasetFilters, embeddingData,
-            features, groupBy, markerOpacity, datasetFilter,
+            categoricalNames, chartSize, numberOfBinsUI, interpolator, binValues, binSummary, embeddings, classes, embeddingData,
+            searchTokens, markerOpacity, datasetFilter, datasetFilters,
             featureSummary, shape, nObsSelected, globalFeatureSummary, unselectedMarkerOpacity, dataset,
-            handleColorChange, handleMeasureFilterUpdated, handleDimensionFilterUpdated, pointSize
+            handleColorChange, handleNameChange, handleMeasureFilterUpdated, handleDimensionFilterUpdated, pointSize,
+            combineDatasetFilters, selection
         } = this.props;
 
+        let currentDatasetFilters = getDatasetFilterArray(datasetFilter);
+
+        const datasetFilterKeys = [];
+        let isBrushing = false;
+        currentDatasetFilters.forEach(f => {
+            if (typeof f[0] === 'object') {
+                isBrushing = true;
+            } else {
+                datasetFilterKeys.push(f[0]);
+            }
+        });
+        datasetFilterKeys.sort((a, b) => {
+            a = a.toLowerCase();
+            b = b.toLowerCase();
+            return a < b ? -1 : (a === b ? 0 : 1);
+        });
+        if (isBrushing) {
+            datasetFilterKeys.push('selection');
+        }
 
-        const pointSizeTicks = [{value: 0.25, label: '25%'}, {value: 0.5, label: '50%'}, {
-            value: 0.75,
-            label: '75%'
-        }, {value: 1, label: '100%'}, {value: 1.5, label: '150%'}, {value: 2, label: '200%'}, {
-            value: 3,
-            label: '300%'
-        }, {value: 4, label: '400%'}];
         // for filters we only need one embedding trace per feature
         const traceNames = new Set();
         const filterTraces = [];
@@ -246,48 +335,58 @@ class EmbedForm extends React.PureComponent {
                 filterTraces.push(trace);
             }
         });
+        filterTraces.sort((a, b) => {
+            a = a.name.toLowerCase();
+            b = b.name.toLowerCase();
+            return a < b ? -1 : (a === b ? 0 : 1);
+        });
         let savedDatasetFilter = this.props.savedDatasetFilter;
         if (savedDatasetFilter == null) {
             savedDatasetFilter = {};
         }
-        const availableFeatures = dataset == null ? [] : dataset.features;
+        const splitTokens = splitSearchTokens(searchTokens);
+        const featureOptions = dataset == null ? [] : dataset.features;
+        const markers = dataset == null || dataset.markers == null ? {} : dataset.markers;
+        const featureSetOptions = [];
+
+        for (const group in markers) {
+            const groupMarkers = markers[group];
+            for (const name in groupMarkers) {
+                featureSetOptions.push({group: group, text: name});
+            }
+        }
+
+        featureSetOptions.sort((item1, item2) => {
+            let a = item1.group.toLowerCase();
+            let b = item2.group.toLowerCase();
+            if (a !== b) {
+                return a < b ? -1 : 1;
+            }
+            a = item1.text.toLowerCase();
+            b = item2.text.toLowerCase();
+            const aNumber = parseFloat(a);
+            const bNumber = parseFloat(b);
+            if (!isNaN(aNumber) && !isNaN(bNumber)) {
+                a = aNumber;
+                b = bNumber;
+            }
+            return a < b ? -1 : (a === b ? 0 : 1);
+        });
+
         const availableEmbeddings = dataset == null ? [] : dataset.embeddings;
+        const embeddingKeys = embeddings.map(e => getEmbeddingKey(e));
         const isSummarized = dataset == null ? false : dataset.precomputed != null;
         const obsCat = dataset == null ? [] : dataset.obsCat;
         const obs = dataset == null ? [] : dataset.obs;
-        const summaryOptions = [
-            {value: 'max', label: 'Maximum'},
-            {value: 'mean', label: 'Mean'},
-            {value: 'sum', label: 'Sum'}];
-        let featureValue = features.concat(groupBy);
-        featureValue = featureValue.map(item => {
-            return {label: item, value: item};
-        });
-        let metadataOptions = obs.map(item => {
-            return {label: item, value: item};
-        });
-        metadataOptions = metadataOptions.concat(obsCat.map(item => {
-            return {label: item, value: item, categorical: true};
-        }));
-        metadataOptions.sort((a, b) => {
-            a = a.label.toLowerCase();
-            b = b.label.toLowerCase();
-            return a < b ? -1 : (a == b ? 0 : 1);
+
+        let annotationOptions = obs.concat(obsCat);
+        annotationOptions.sort((a, b) => {
+            a = a.toLowerCase();
+            b = b.toLowerCase();
+            return a < b ? -1 : (a === b ? 0 : 1);
         });
 
-        let allOptions = [{label: 'Annotations', options: metadataOptions}, {
-            label: 'Variables',
-            options: availableFeatures.map(item => {
-                return {label: item, value: item};
-            })
-        }];
-        let defaultOptions = [{
-            label: 'Annotations', options: metadataOptions
-        }, {
-            label: 'Variables',
-            options: [{isDisabled: true, label: 'Type to search', value: ''}]
-        }];
-        const embeddingKeys = embeddings.map(e => getEmbeddingKey(e));
+
         return (
             <div className={classes.root}>
                 <FormControl className={classes.formControl}>
@@ -310,29 +409,114 @@ class EmbedForm extends React.PureComponent {
                         ))}
                     </Select>
                 </FormControl>
-                <FormControl className={classes.formControl} style={{zIndex: 2000}}>
-                    <AutocompleteSelect label="Features" options={allOptions}
-                                        defaultOptions={defaultOptions} value={featureValue}
-                                        onChange={this.props.handleFeatures}
-                                        isMulti={true}/>
+                <FormControl className={classes.formControl}>
+
+                    {/*<AutocompleteSelect label="Features" options={allOptions}*/}
+                    {/*                    defaultOptions={defaultOptions} value={featureValue}*/}
+                    {/*                    onChange={this.props.handleFeatures}*/}
+                    {/*                    helperText={'Enter or paste list'}*/}
+                    {/*                    isMulti={true}/>*/}
+                    <AutocompleteVirtualized label={"Features"} options={featureOptions} value={splitTokens.X}
+                                             onChange={this.onFeaturesChange}/>
                 </FormControl>
 
-                <ExpansionPanel defaultExpanded>
-                    <ExpansionPanelSummary
+                <FormControl className={classes.formControl}>
+
+                    {/*<AutocompleteSelect label="Features" options={allOptions}*/}
+                    {/*                    defaultOptions={defaultOptions} value={featureValue}*/}
+                    {/*                    onChange={this.props.handleFeatures}*/}
+                    {/*                    helperText={'Enter or paste list'}*/}
+                    {/*                    isMulti={true}/>*/}
+                    <AutocompleteVirtualized label={"Observations"} options={annotationOptions}
+                                             value={splitTokens.obs.concat(splitTokens.obsCat)}
+                                             onChange={this.onObservationsChange}/>
+                </FormControl>
+
+                {featureSetOptions.length > 0 && <FormControl className={classes.formControl}>
+
+                    <AutocompleteVirtualized label={"Sets"} options={featureSetOptions}
+                                             value={splitTokens.featureSets}
+                                             onChange={this.onFeatureSetsChange} groupBy={true}/>
+                </FormControl>}
+
+                <Accordion defaultExpanded>
+                    <AccordionPanelSummary
                         aria-controls="summary-content"
                         id="summary-header"
                     >
-                        <div>Filters</div>
-                    </ExpansionPanelSummary>
-                    <ExpansionPanelDetails>
+                        <div>Filter</div>
+                    </AccordionPanelSummary>
+                    <AccordionPanelDetails>
                         <div style={{marginLeft: 10}}>
+                            <div>
+                                <Grid component="label" container alignItems="center" spacing={0}>
+                                    <Grid item>AND</Grid>
+                                    <Grid item>
+                                        <Switch
+                                            size="small"
+                                            checked={combineDatasetFilters === 'or'}
+                                            onChange={this.handleCombineDatasetFilters}
+                                        />
+                                    </Grid>
+                                    <Grid item>OR</Grid>
+                                </Grid>
+                            </div>
+                            {datasetFilterKeys.length > 0 && !isNaN(selection.count) &&
+                            <React.Fragment>
+                                <div style={{marginBottom: 2}}>
+                                    {intFormat(selection.count) + " / " + intFormat(dataset.shape[0]) + ": "}
+                                    {datasetFilterKeys.map(key => {
+                                        return <Chip
+                                            size="small"
+                                            onDelete={() => {
+                                                this.onDatasetFilterChipDeleted(key);
+                                            }}
+                                            style={{marginRight: 2, verticalAlign: 'bottom'}}
+                                            key={key}
+                                            label={key}
+
+                                        />;
+                                    })}
+                                    <Divider/>
+                                    <Tooltip title={"Clear All"}>
+                                        <IconButton size={'small'} disabled={datasetFilterKeys.length === 0}
+                                                    onClick={this.onDatasetFilterCleared}><HighlightOffIcon/></IconButton>
+                                    </Tooltip>
+                                    <Tooltip title={"Save Filter"}>
+                                        <IconButton size={'small'}  disabled={datasetFilterKeys.length === 0}
+                                                    onClick={this.onDatasetFilterSaved}><SaveIcon/></IconButton>
+                                    </Tooltip>
+                                    <Tooltip title={"Download Selected IDs"}>
+                                        <IconButton size={'small'}  disabled={datasetFilterKeys.length === 0}
+                                                    onClick={this.handleSelectedCellsClick}><CloudDownloadIcon/></IconButton>
+                                    </Tooltip>
+                                    {/*<Tooltip title={"Compute Markers"}>*/}
+                                    {/*    <Button color="primary" variant="outlined"*/}
+                                    {/*            disabled={datasetFilterKeys.length === 0}*/}
+                                    {/*            size={"small"}*/}
+                                    {/*            onClick={this.handleDiffExp}>Markers</Button>*/}
+                                    {/*</Tooltip>*/}
+                                    <Divider/>
+                                </div>
+
+                            </React.Fragment>
+                            }
+
+
                             {filterTraces.map(traceInfo =>
+
                                 traceInfo.continuous ?
                                     <ColorSchemeLegendWrapper
                                         key={traceInfo.name}
                                         width={140}
                                         showColorScheme={false}
                                         height={30}
+                                        style={{
+                                            paddingBottom: 3,
+                                            paddingTop: 3,
+                                            display: 'block',
+                                            borderBottom: '1px solid rgba(0, 0, 0, 0.12)'
+                                        }}
                                         handleUpdate={handleMeasureFilterUpdated}
                                         datasetFilter={datasetFilter}
                                         scale={traceInfo.colorScale}
@@ -345,9 +529,17 @@ class EmbedForm extends React.PureComponent {
                                     /> :
                                     <CategoricalLegend
                                         key={traceInfo.name}
+                                        style={{
+                                            paddingBottom: 3,
+                                            paddingTop: 3,
+                                            display: 'block',
+                                            borderBottom: '1px solid rgba(0, 0, 0, 0.12)'
+                                        }}
                                         datasetFilter={datasetFilter}
                                         handleClick={handleDimensionFilterUpdated}
                                         handleColorChange={handleColorChange}
+                                        handleNameChange={handleNameChange}
+                                        categoricalNames={categoricalNames}
                                         name={traceInfo.name}
                                         scale={traceInfo.colorScale}
                                         maxHeight={300}
@@ -358,18 +550,19 @@ class EmbedForm extends React.PureComponent {
                                         featureSummary={featureSummary}/>
                             )}
 
-                        </div>
-                    </ExpansionPanelDetails>
-                </ExpansionPanel>
 
-                <ExpansionPanel defaultExpanded>
-                    <ExpansionPanelSummary
-                        aria-controls="chart-options-content"
-                        id="chart-options-header"
+                        </div>
+                    </AccordionPanelDetails>
+                </Accordion>
+
+                <Accordion defaultExpanded>
+                    <AccordionPanelSummary
+                        aria-controls="view-options-content"
+                        id="view-options-header"
                     >
-                        <div>Chart Options</div>
-                    </ExpansionPanelSummary>
-                    <ExpansionPanelDetails>
+                        <div>View Options</div>
+                    </AccordionPanelSummary>
+                    <AccordionPanelDetails>
                         <div>
 
 
@@ -395,7 +588,7 @@ class EmbedForm extends React.PureComponent {
                                     onChange={this.onPointSizeChange}
                                     value={pointSize}
                                     multiple={false}>
-                                    {pointSizeTicks.map(item => (
+                                    {pointSizeOptions.map(item => (
                                         <MenuItem key={item.label} value={item.value}>
                                             <ListItemText primary={item.label}/>
                                         </MenuItem>
@@ -403,12 +596,27 @@ class EmbedForm extends React.PureComponent {
                                 </Select>
                             </FormControl>
 
+                            <FormControl className={classes.formControl}>
+                                <InputLabel htmlFor="chart_size">Gallery Chart Size</InputLabel>
+                                <Select
+                                    className={classes.select}
+                                    input={<Input id="chart_size"/>}
+                                    onChange={this.onChartSizeChange}
+                                    value={chartSize}
+                                    multiple={false}>
+                                    {gallerySizeOptions.map(item => (
+                                        <MenuItem key={item.label} value={item.value}>
+                                            <ListItemText primary={item.label}/>
+                                        </MenuItem>
+                                    ))}
+                                </Select>
+                            </FormControl>
 
                             <FormControl className={classes.formControl}>
-                                <InputLabel htmlFor="color-scheme">Color Scheme</InputLabel>
+                                <InputLabel htmlFor="color-scheme">Continuous Color Scale</InputLabel>
                                 <ColorSchemeSelector/>
                             </FormControl>
-                            {<FormControlLabel
+                            <div><FormControlLabel
                                 control={
                                     <Switch
                                         checked={interpolator.reversed || false}
@@ -417,9 +625,9 @@ class EmbedForm extends React.PureComponent {
                                     />
                                 }
                                 label="Reverse Colors"
-                            />}
+                            /></div>
 
-                            {!isSummarized && <FormControlLabel
+                            {!isSummarized && <div><FormControlLabel
                                 control={
                                     <Switch
                                         checked={binValues}
@@ -428,7 +636,7 @@ class EmbedForm extends React.PureComponent {
                                     />
                                 }
                                 label="Bin Plot"
-                            />}
+                            /></div>}
 
                             {!isSummarized && binValues &&
                             <TextField max="1000" min="20" step="100" onKeyPress={this.onNumberOfBinsKeyPress}
@@ -467,16 +675,16 @@ class EmbedForm extends React.PureComponent {
 
                         </div>
 
-                    </ExpansionPanelDetails>
-                </ExpansionPanel>
-                <ExpansionPanel defaultExpanded>
-                    <ExpansionPanelSummary
+                    </AccordionPanelDetails>
+                </Accordion>
+                <Accordion defaultExpanded>
+                    <AccordionPanelSummary
                         aria-controls="filter-options-content"
                         id="filter-options-header"
                     >
                         <div>Saved Filters</div>
-                    </ExpansionPanelSummary>
-                    <ExpansionPanelDetails>
+                    </AccordionPanelSummary>
+                    <AccordionPanelDetails>
                         <div>
                             {datasetFilters.length === 0 &&
                             <Box color="text.secondary">No saved filters</Box>}
@@ -497,8 +705,8 @@ class EmbedForm extends React.PureComponent {
                                 <Button onClick={this.props.handleExportDatasetFilters}>Export Filters</Button>
                             </div>}
                         </div>
-                    </ExpansionPanelDetails>
-                </ExpansionPanel>
+                    </AccordionPanelDetails>
+                </Accordion>
             </div>
         );
     }
@@ -506,9 +714,7 @@ class EmbedForm extends React.PureComponent {
 
 const mapStateToProps = state => {
     return {
-
-
-        datasetFilter: state.datasetFilter,
+        categoricalNames: state.categoricalNames,
         featureSummary: state.featureSummary,
         shape: state.dataset.shape,
         nObsSelected: state.selection.count,
@@ -518,7 +724,6 @@ const mapStateToProps = state => {
         binSummary: state.binSummary,
         numberOfBins: state.numberOfBins,
         numberOfBinsUI: state.numberOfBinsUI,
-        datasetFilters: state.datasetFilters,
         embeddingData: state.embeddingData,
         embeddingChartSize: state.embeddingChartSize,
         interpolator: state.interpolator,
@@ -526,20 +731,44 @@ const mapStateToProps = state => {
         pointSize: state.pointSize,
         savedDatasetFilter: state.savedDatasetFilter,
         embeddings: state.embeddings,
-        features: state.features,
-        groupBy: state.groupBy,
+        searchTokens: state.searchTokens,
         unselectedMarkerOpacity: state.unselectedMarkerOpacityUI,
-        unselectedMarkerSize: state.unselectedMarkerSizeUI
+        combineDatasetFilters: state.combineDatasetFilters,
+        datasetFilter: state.datasetFilter,
+        datasetFilters: state.datasetFilters,
+        selection: state.selection,
+        chartSize: state.chartSize
     };
 };
 const mapDispatchToProps = (dispatch, ownProps) => {
     return {
+        handleDialog: (value) => {
+            dispatch(setDialog(value));
+        },
+        handleChartSize: (value) => {
+            dispatch(setChartSize(value));
+        },
+        handleDiffExp: (value) => {
+            dispatch(diffExp(value));
+        },
+        handleCombineDatasetFilters: (value) => {
+            dispatch(setCombineDatasetFilters(value));
+        },
+        downloadSelectedIds: () => {
+            dispatch(downloadSelectedIds());
+        },
+        removeDatasetFilter: (filter) => {
+            dispatch(removeDatasetFilter(filter));
+        },
         handleDimensionFilterUpdated: (e) => {
             dispatch(handleDimensionFilterUpdated(e));
         },
         handleColorChange: (e) => {
             dispatch(handleColorChange(e));
         },
+        handleNameChange: (e) => {
+            dispatch(handleCategoricalNameChange(e));
+        },
         handleMeasureFilterUpdated: (e) => {
             dispatch(handleMeasureFilterUpdated(e));
         },
@@ -549,7 +778,6 @@ const mapDispatchToProps = (dispatch, ownProps) => {
         onDeselect: (e) => {
             dispatch(handleBrushFilterUpdated(e));
         },
-
         handleEmbeddings: value => {
             dispatch(setSelectedEmbedding(value));
         },
@@ -583,8 +811,8 @@ const mapDispatchToProps = (dispatch, ownProps) => {
         handleBinValues: value => {
             dispatch(setBinValues(value));
         },
-        handleFeatures: value => {
-            dispatch(setFeatures(value == null ? [] : value));
+        handleSearchTokens: (value, type) => {
+            dispatch(setSearchTokens(value == null ? [] : value, type));
         },
         handleOpenDatasetFilter: value => {
             dispatch(openDatasetFilter(value));
