@@ -3,11 +3,15 @@ import {combineReducers} from 'redux';
 import {
     ADD_DATASET,
     DELETE_DATASET,
+    DIFF_EXP_RESULTS,
     getTraceKey,
     RESTORE_VIEW,
     SET_BIN_SUMMARY,
     SET_BIN_VALUES,
     SET_CATEGORICAL_COLOR,
+    SET_CATEGORICAL_NAME,
+    SET_CHART_OPTIONS,
+    SET_CHART_SIZE,
     SET_COMBINE_DATASET_FILTERS,
     SET_DATASET,
     SET_DATASET_CHOICES,
@@ -19,10 +23,7 @@ import {
     SET_EMAIL,
     SET_EMBEDDING_DATA,
     SET_FEATURE_SUMMARY,
-    SET_FEATURES,
-    SET_FEATURES_UI,
     SET_GLOBAL_FEATURE_SUMMARY,
-    SET_GROUP_BY,
     SET_INTERPOLATOR,
     SET_LOADING,
     SET_LOADING_APP,
@@ -30,9 +31,11 @@ import {
     SET_MARKER_OPACITY_UI,
     SET_MESSAGE,
     SET_NUMBER_OF_BINS,
-    SET_POINT_SIZE,
     SET_NUMBER_OF_BINS_UI,
+    SET_POINT_SIZE,
     SET_PRIMARY_TRACE_KEY,
+    SET_SEARCH_TOKENS,
+    SET_SELECTED_DOT_PLOT_DATA,
     SET_SELECTED_EMBEDDING,
     SET_SELECTION,
     SET_SERVER_INFO,
@@ -42,7 +45,7 @@ import {
     SET_USER,
     UPDATE_DATASET,
 } from '../actions';
-import {getInterpolator, updateTraceColors} from '../PlotUtil';
+import {getInterpolator, updateTraceColors} from '../util';
 
 export const DEFAULT_BIN_SUMMARY = 'max';
 export const DEFAULT_NUMBER_BINS = 500;
@@ -56,51 +59,53 @@ const DEFAULT_INTERPOLATOR_OBJ = {
     value: getInterpolator(DEFAULT_INTERPOLATOR)
 };
 
-function features(state = [], action) {
+const DEFAULT_CHART_OPTIONS = {
+    animating: false, dragmode: 'pan', editSelection: false, showLabels: false,
+    showAxis: true, showFog: true, darkMode: false
+};
+
+function chartSize(state = 500, action) {
     switch (action.type) {
-        case SET_FEATURES:
+        case SET_CHART_SIZE:
             return action.payload;
-        case SET_DATASET:
-            return [];
-        case RESTORE_VIEW:
-            return action.payload.features || [];
         default:
             return state;
     }
 }
 
-function featuresUI(state = [], action) {
+/**
+ *
+ * @param state Array of value, type where type can be X, obs, or obsCat
+ * @param action
+ * @returns {*|*[]}
+ */
+function searchTokens(state = [], action) {
     switch (action.type) {
-        case SET_FEATURES:
-        case SET_FEATURES_UI:
+        case SET_SEARCH_TOKENS:
             return action.payload;
         case SET_DATASET:
             return [];
         case RESTORE_VIEW:
-            return action.payload.features || [];
+            return action.payload.q || [];
         default:
             return state;
     }
 }
 
 
-function groupBy(state = [], action) {
+function chartOptions(state = DEFAULT_CHART_OPTIONS, action) {
     switch (action.type) {
-        case SET_GROUP_BY:
-            return action.payload;
-        case SET_DATASET:
-            return [];
-        case RESTORE_VIEW:
-            return action.payload.groupBy || [];
+        case SET_CHART_OPTIONS:
+            return Object.assign({}, action.payload);
         default:
             return state;
     }
 }
 
-
 function dataset(state = null, action) {
     switch (action.type) {
         case SET_DATASET:
+            document.title = action.payload == null ? 'Cirro' : action.payload.name + ' - Cirro';
             return action.payload;
         default:
             return state;
@@ -121,6 +126,18 @@ function embeddings(state = [], action) {
     }
 }
 
+function diffExpResults(state = null, action) {
+    switch (action.type) {
+        case DIFF_EXP_RESULTS:
+            return action.payload;
+        case SET_DATASET:
+            return null;
+        default:
+            return state;
+    }
+}
+
+
 function binSummary(state = DEFAULT_BIN_SUMMARY, action) {
     switch (action.type) {
         case SET_BIN_SUMMARY:
@@ -380,8 +397,44 @@ function dialog(state = null, action) {
     }
 }
 
+// [{
+//     "categories": array,
+//     "name": str,
+//     "fractionRange":[number],
+//     "meanRange":[number]
+//     "values": [{
+//         "name": str,
+//         "fractionExpressed": array
+//         "mean": array,
+//         "active": bool
+//     }]
+// }]
+
+function updateDotPlotDataRange(data) {
+    data.forEach(categoryItem => {
+        let fractionRange = [Number.MAX_VALUE, -Number.MAX_VALUE];
+        let meanRange = [Number.MAX_VALUE, -Number.MAX_VALUE];
+        categoryItem.values.forEach(feature => {
+            for (let i = 0, n = feature.mean.length; i < n; i++) {
+                fractionRange[0] = Math.min(feature.fractionExpressed[i], fractionRange[0]);
+                fractionRange[1] = Math.max(feature.fractionExpressed[i], fractionRange[1]);
+                meanRange[0] = Math.min(feature.mean[i], meanRange[0]);
+                meanRange[1] = Math.max(feature.mean[i], meanRange[1]);
+            }
+        });
+        categoryItem.meanRange = meanRange;
+        categoryItem.fractionRange = fractionRange;
+    });
+}
+
 function dotPlotData(state = [], action) {
     switch (action.type) {
+        case SET_CATEGORICAL_NAME:
+            for (let i = 0; i < state.length; i++) {
+                let item = state[i];
+                state[i] = Object.assign({}, item);
+            }
+            return state.slice();
         case SET_DOT_PLOT_SORT_ORDER:
             const name = action.payload.name;
             const sortBy = action.payload.value;
@@ -395,6 +448,37 @@ function dotPlotData(state = [], action) {
             }
             return state.slice();
         case SET_DOT_PLOT_DATA:
+            updateDotPlotDataRange(action.payload);
+            return action.payload;
+        case SET_DATASET:
+            return [];
+        default:
+            return state;
+    }
+}
+
+function selectedDotPlotData(state = [], action) {
+    switch (action.type) {
+        case SET_CATEGORICAL_NAME:
+            for (let i = 0; i < state.length; i++) {
+                let item = state[i];
+                state[i] = Object.assign({}, item);
+            }
+            return state.slice();
+        case SET_DOT_PLOT_SORT_ORDER:
+            const name = action.payload.name;
+            const sortBy = action.payload.value;
+            for (let i = 0; i < state.length; i++) {
+                let item = state[i];
+                if (item.name === name) {
+                    item.sortBy = sortBy;
+                    state[i] = Object.assign({}, item);
+                    break;
+                }
+            }
+            return state.slice();
+        case SET_SELECTED_DOT_PLOT_DATA:
+            updateDotPlotDataRange(action.payload);
             return action.payload;
         case SET_DATASET:
             return [];
@@ -437,6 +521,28 @@ function combineDatasetFilters(state = 'and', action) {
     }
 }
 
+// category -> value -> newValue
+function categoricalNames(state = {}, action) {
+    switch (action.type) {
+        case SET_CATEGORICAL_NAME:
+            let category = state[action.payload.name];
+            if (category === undefined) {
+                category = {};
+                state[action.payload.name] = category;
+            }
+            if (action.payload.value == null || action.payload.value === '') {
+                delete category[action.payload.oldValue];
+            } else {
+                category[action.payload.oldValue] = action.payload.value;
+            }
+            return Object.assign({}, state);
+        case SET_DATASET:
+            return {};
+        default:
+            return state;
+    }
+}
+
 
 // each item has  data (list of traces, each trace has x, y, etc.), layout
 function embeddingData(state = [], action) {
@@ -456,6 +562,7 @@ function embeddingData(state = [], action) {
                 }
             });
             return state.slice();
+
         case SET_INTERPOLATOR:
             // update colors for existing continuous traces
 
@@ -531,6 +638,9 @@ function interpolator(state = DEFAULT_INTERPOLATOR_OBJ, action) {
 export default combineReducers({
     binSummary,
     binValues,
+    categoricalNames,
+    chartOptions,
+    chartSize,
     combineDatasetFilters,
     datasetFilter,
     datasetFilters,
@@ -538,14 +648,12 @@ export default combineReducers({
     datasetChoices,
     dialog,
     dotPlotData,
+    diffExpResults,
     email,
     embeddingData,
     embeddings,
-    features,
     featureSummary,
-    featuresUI,
     globalFeatureSummary,
-    groupBy,
     interpolator,
     loading,
     loadingApp,
@@ -556,7 +664,9 @@ export default combineReducers({
     message,
     pointSize,
     primaryTraceKey,
+    searchTokens,
     selection,
+    selectedDotPlotData,
     serverInfo,
     tab,
     unselectedMarkerOpacity,
